//importing required modules
import {
  scaleLinear,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  symbols,
  symbol,
  symbolCross,
  symbolDiamond,
  symbolSquare,
  schemeCategory10,
} from 'd3';
//here we export scatter plot to html
export const scatterPlot = () => {
  let width;
  let height;
  let data;
  let xValue;
  let yValue;
  let margin;
  let radius;
//instance of the chart
  const colr = ['Orange','#56486c','#f51e05'];
  const speciesValue = (d) => d.species;
  const my = (selection) => {
    //x scale function
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    //y scale function
    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
      species: d.species,
    }));
    // create a shape separator where it will take the value from species column to assign a shape
    // different value will have different shape. 
    var shape = scaleOrdinal(data.map(d => d.species), symbols.map(s => symbol().type(s)()))
    //Different species will have different color which are picked from colr array.
    var color = scaleOrdinal(data.map(d => {d.species}), colr)
    
    
    const circles = selection
      .selectAll('.dots')
      .data(marks)
    // adding some actions
      .join(
      // drawing the data point on the graph
      enter => enter.append("path").attr("class", "dots"),
      // updating the data point
      update => update,
      // remove the data point so that new data point will be added without overlapping on the previous dataset
      exit => exit.remove()
      )
    
    // drawing shape and color  
      .attr('d', d => shape(d.species))
      .attr('fill', d => color(d.species))
      .attr('transform', (d) => `translate(${d.x},${d.y})`);

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left},10)`)
      .call(axisLeft(y));

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0,${height - margin.bottom})`
      )
      .call(axisBottom(x));
   
    
  };
//accessors : they will be used as getters and setters
  my.width = function (_) {
    return arguments.length ? ((width = +_), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = +_), my) : height;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = +_), my) : radius;
  };

  return my;
};
